###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $

__doc__ = 'Value checker and processor'
__version__ = '$Revision 1.0 $'

import Globals
import ZPublisher
import integer
import float
import required
import string

value_checkers = {}

message_dialogue = Globals.HTMLFile('message_dialogue', globals())

http_request_http_request_process_inputs = ZPublisher.HTTPRequest.HTTPRequest.processInputs


def register_value_checker(name, module):
	"""This let's us add value checkers from other modules."""

	value_checkers[name] = module


register_value_checker('integer', integer)
register_value_checker('required', required)
register_value_checker('float', float)


def processInputs(self):
	"""Method that hooks itself onto the standard form data processing."""

	# We don't want to disturb anybody

	http_request_http_request_process_inputs(self)

	if self.form.has_key('__ignore_conversion_error'):

		ignore_conversion_error = 1

	else:

		ignore_conversion_error = 0

	# How can we can ahold of the object being published?
	# Seems we have to do something that they do in the ZPublisher.Publish module..
	# Hmmmm.... =]

	published_object = None


	string_find = string.find
	string_split = string.split

	self.value_checker_exception_stack = []

	form_update = {}

	print self.form.keys()

	for key in self.form.keys():

		print 'prosessing key', key

		if string_find(key, ';') > -1:

			splits = string_split(key, ';')

			field_name = splits[0]

			for value_checker in splits[1:]:

				value = self.form[key]

				if value_checker in value_checkers.keys():

					value_checker = value_checkers[value_checker]

					try:

						form_update[field_name] = value_checker.convert(value)

					except ValueError, value:

						if ignore_conversion_error:

							form_update[field_name] = value
							continue

						self.value_checker_exception_stack.append(
							{
								'field_name':field_name,
								'value':value,
								'help':value_checker.help
							})

						# We need to add some logic so that the user
						# doesn't have to go back, fill in the wrong value
						# and then go forward again, to then go back,
						# etc...

						break

				else:

					# We're daft about unknown value checkers

					print 'Unknown value checker/converter:', value_checker

	if self.value_checker_exception_stack:

		# Why does the message_dialogue mess up the exception stack anyway?

		if hasattr(published_object, 'standard_value_error_message'):

			published_object.standard_value_error_message(None, REQUEST=self, RESPONSE=self.RESPONSE)

		else:

			self.RESPONSE.write(message_dialogue(None, REQUEST=self, RESPONSE=self.RESPONSE))


ZPublisher.HTTPRequest.HTTPRequest.processInputs = processInputs
